@@ -23,11 +23,10 @@
 #include <QScrollBar>
 #include <QSize>
 
-Overview::Overview(const QSharedPointer<Options>& pOptions)
+Overview::Overview()
 //: QWidget( pParent, 0, Qt::WNoAutoErase )
 {
     m_pDiff3LineList = nullptr;
-    m_pOptions = pOptions;
     mOverviewMode = e_OverviewMode::eOMNormal;
     m_nofLines = 1;
     setUpdatesEnabled(false);
@@ -55,15 +54,18 @@ void Overview::slotRedraw()
     update();
 }
 
-void Overview::setRange(qint32 firstLine, qint32 pageHeight)
+void Overview::setRange(LineRef firstLine, qint32 pageHeight)
 {
+    assert(firstLine.isValid());
     m_firstLine = firstLine;
     m_pageHeight = pageHeight;
     update();
 }
-void Overview::setFirstLine(qint32 firstLine)
+
+void Overview::setFirstLine(LineRef firstLine)
 {
     QScrollBar* scrollBar = qobject_cast<QScrollBar*>(sender());
+    assert(firstLine.isValid());
 
     if(Q_UNLIKELY(scrollBar == nullptr))
     {
@@ -128,18 +130,18 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
     {
         const Diff3Line& d3l = *i;
         qint32 y = h * (line + 1) / nofLines;
-        MergeLine lMergeLine;
+        MergeBlock lMergeBlock;
         e_MergeDetails md;
         bool bConflict;
         bool bLineRemoved;
 
-        lMergeLine.mergeOneLine(d3l, bLineRemoved, !KDiff3App::isTripleDiff());
-        md = lMergeLine.details();
-        bConflict = lMergeLine.isConflict();
+        lMergeBlock.mergeOneLine(d3l, bLineRemoved, !KDiff3App::isTripleDiff());
+        md = lMergeBlock.details();
+        bConflict = lMergeBlock.isConflict();
 
-        QColor c = m_pOptions->backgroundColor();
+        QColor c = gOptions->backgroundColor();
         bool bWhiteSpaceChange = false;
-        //if( bConflict )  c=m_pOptions->conflictColor();
+        //if( bConflict )  c=gOptions->conflictColor();
         //else
         switch(eOverviewMode)
         {
@@ -148,13 +150,13 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                 {
                     case e_MergeDetails::eDefault:
                     case e_MergeDetails::eNoChange:
-                        c = m_pOptions->backgroundColor();
+                        c = gOptions->backgroundColor();
                         break;
 
                     case e_MergeDetails::eBAdded:
                     case e_MergeDetails::eBDeleted:
                     case e_MergeDetails::eBChanged:
-                        c = bConflict ? m_pOptions->conflictColor() : m_pOptions->bColor();
+                        c = bConflict ? gOptions->conflictColor() : gOptions->bColor();
                         bWhiteSpaceChange = d3l.isEqualAB() || (d3l.isWhiteLine(e_SrcSelector::A) && d3l.isWhiteLine(e_SrcSelector::B));
                         break;
 
@@ -162,7 +164,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                     case e_MergeDetails::eCDeleted:
                     case e_MergeDetails::eCChanged:
                         bWhiteSpaceChange = d3l.isEqualAC() || (d3l.isWhiteLine(e_SrcSelector::A) && d3l.isWhiteLine(e_SrcSelector::C));
-                        c = bConflict ? m_pOptions->conflictColor() : m_pOptions->cColor();
+                        c = bConflict ? gOptions->conflictColor() : gOptions->cColor();
                         break;
 
                     case e_MergeDetails::eBCChanged:         // conflict
@@ -172,7 +174,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                     case e_MergeDetails::eCChanged_BDeleted: // conflict
                     case e_MergeDetails::eBCAdded:           // conflict
                     case e_MergeDetails::eBCAddedAndEqual:   // possible conflict
-                        c = m_pOptions->conflictColor();
+                        c = gOptions->conflictColor();
                         break;
                 }
                 break;
@@ -186,7 +188,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                     case e_MergeDetails::eCChanged:
                         break;
                     default:
-                        c = m_pOptions->conflictColor();
+                        c = gOptions->conflictColor();
                         bWhiteSpaceChange = d3l.isEqualAB() || (d3l.isWhiteLine(e_SrcSelector::A) && d3l.isWhiteLine(e_SrcSelector::B));
                         break;
                 }
@@ -201,7 +203,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                     case e_MergeDetails::eBChanged:
                         break;
                     default:
-                        c = m_pOptions->conflictColor();
+                        c = gOptions->conflictColor();
                         bWhiteSpaceChange = d3l.isEqualAC() || (d3l.isWhiteLine(e_SrcSelector::A) && d3l.isWhiteLine(e_SrcSelector::C));
                         break;
                 }
@@ -216,7 +218,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
                     case e_MergeDetails::eBCAddedAndEqual:
                         break;
                     default:
-                        c = m_pOptions->conflictColor();
+                        c = gOptions->conflictColor();
                         bWhiteSpaceChange = d3l.isEqualBC() || (d3l.isWhiteLine(e_SrcSelector::B) && d3l.isWhiteLine(e_SrcSelector::C));
                         break;
                 }
@@ -230,26 +232,26 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
         {
             if(!d3l.getLineA().isValid() && d3l.getLineB().isValid())
             {
-                c = m_pOptions->aColor();
+                c = gOptions->aColor();
                 x2 = w / 2;
                 w2 = x2;
             }
             if(d3l.getLineA().isValid() && !d3l.getLineB().isValid())
             {
-                c = m_pOptions->bColor();
+                c = gOptions->bColor();
                 w2 = w / 2;
             }
         }
 
-        if(!bWhiteSpaceChange || m_pOptions->m_bShowWhiteSpace)
+        if(!bWhiteSpaceChange || gOptions->m_bShowWhiteSpace)
         {
             // Make sure that lines with conflict are not overwritten.
-            if(c == m_pOptions->conflictColor())
+            if(c == gOptions->conflictColor())
             {
                 p.fillRect(x2 + 1, oldY, w2, std::max(1, y - oldY), bWhiteSpaceChange ? QBrush(c, Qt::Dense4Pattern) : QBrush(c));
                 oldConflictY = oldY;
             }
-            else if(c != m_pOptions->backgroundColor() && oldY > oldConflictY)
+            else if(c != gOptions->backgroundColor() && oldY > oldConflictY)
             {
                 p.fillRect(x2 + 1, oldY, w2, std::max(1, y - oldY), bWhiteSpaceChange ? QBrush(c, Qt::Dense4Pattern) : QBrush(c));
             }
@@ -258,7 +260,7 @@ void Overview::drawColumn(QPainter& p, e_OverviewMode eOverviewMode, qint32 x, q
         oldY = y;
 
         ++line;
-        if(m_pOptions->wordWrapOn())
+        if(gOptions->wordWrapOn())
         {
             ++wrapLineIdx;
             if(wrapLineIdx >= d3l.linesNeededForDisplay())
@@ -283,13 +285,13 @@ void Overview::paintEvent(QPaintEvent*)
     const auto dpr = devicePixelRatioF();
     if(m_pixmap.size() != size() * dpr)
     {
-        m_nofLines = m_pDiff3LineList->numberOfLines(m_pOptions->wordWrapOn());
+        m_nofLines = m_pDiff3LineList->numberOfLines(gOptions->wordWrapOn());
 
         m_pixmap = QPixmap(size() * dpr);
         m_pixmap.setDevicePixelRatio(dpr);
 
         QPainter p(&m_pixmap);
-        p.fillRect(rect(), m_pOptions->backgroundColor());
+        p.fillRect(rect(), gOptions->backgroundColor());
 
         if(!KDiff3App::isTripleDiff() || mOverviewMode == e_OverviewMode::eOMNormal)
         {
