@@ -11,9 +11,9 @@
 #define OVERVIEW_H
 
 #include "LineRef.h"       // for LineRef
-#include "TypeUtils.h"     // for qint32
 
-#include <QSharedPointer>
+#include <memory>
+
 #include <QString>         // for QString
 #include <QPixmap>
 #include <QWidget>
@@ -33,30 +33,29 @@ class Overview : public QWidget
 {
     Q_OBJECT
   public:
-    explicit Overview(const QSharedPointer<Options> &pOptions);
+    Overview();
 
     void init(Diff3LineList* pDiff3LineList);
     void reset();
-    void setRange(qint32 firstLine, qint32 pageHeight);
+    void setRange(LineRef firstLine, qint32 pageHeight);
     void setPaintingAllowed(bool bAllowPainting);
 
     e_OverviewMode getOverviewMode();
 
   public Q_SLOTS:
     void setOverviewMode(e_OverviewMode eOverviewMode);
-    void setFirstLine(qint32 firstLine);
+    void setFirstLine(LineRef firstLine);
     void slotRedraw();
   Q_SIGNALS:
     void setLine(LineRef);
 
   private:
     const Diff3LineList* m_pDiff3LineList;
-    QSharedPointer<Options> m_pOptions;
     LineRef m_firstLine;
     qint32 m_pageHeight;
     QPixmap m_pixmap;
     e_OverviewMode mOverviewMode;
-    qint32 m_nofLines;
+    LineType m_nofLines;
 
     void paintEvent(QPaintEvent* e) override;
     void mousePressEvent(QMouseEvent* e) override;
