@@ -9,7 +9,7 @@
 // clang-format on
 
 #include "kdiff3_shell.h"
-#include "UTF8BOMCodec.h"
+#include "TypeUtils.h"
 #include "version.h"
 
 #include <stdio.h>  // for fileno, stderr
@@ -20,7 +20,7 @@
 #endif
 
 #include <KAboutData>
-#include <KCrash/KCrash>
+#include <KCrash>
 #include <KLocalizedString>
 #include <KMessageBox>
 
@@ -28,7 +28,7 @@
 #include <QCommandLineOption>
 #include <QCommandLineParser>
 #include <QFile>
-#include <QPointer>
+#include <QScopedPointer>
 #include <QStandardPaths>
 #include <QStringList>
 #include <QTextStream>
@@ -46,7 +46,7 @@ void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
             const QString line = ts.readLine();
             if(line.startsWith(u8"IgnorableCmdLineOptions="))
             {
-                const qint32 pos = line.indexOf('=');
+                const QtSizeType pos = line.indexOf('=');
                 if(pos >= 0)
                 {
                     ignorableOptionsLine = line.mid(pos + 1);
@@ -55,25 +55,21 @@ void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
             }
         }
     }
-    //support our own old preferences this is obsolete
-    const QStringList sl = ignorableOptionsLine.split(',');
 
-    if(!sl.isEmpty())
+    const QStringList ignorableOptions = ignorableOptionsLine.split(';');
+
+    for(QString ignorableOption: ignorableOptions)
     {
-        const QStringList ignorableOptions = sl.front().split(';');
-        for(QString ignorableOption: ignorableOptions)
+        ignorableOption.remove('-');
+        if(!ignorableOption.isEmpty())
         {
-            ignorableOption.remove('-');
-            if(!ignorableOption.isEmpty())
+            if(ignorableOption.length() == 1)
             {
-                if(ignorableOption.length() == 1)
-                {
-                    cmdLineParser->addOption(QCommandLineOption({ignorableOption, u8"ignore"}, i18n("Ignored. (User defined.)")));
-                }
-                else
-                {
-                    cmdLineParser->addOption(QCommandLineOption(ignorableOption, i18n("Ignored. (User defined.)")));
-                }
+                cmdLineParser->addOption(QCommandLineOption({ignorableOption, u8"ignore"}, i18n("Ignored. (User defined.)")));
+            }
+            else
+            {
+                cmdLineParser->addOption(QCommandLineOption(ignorableOption, i18n("Ignored. (User defined.)")));
             }
         }
     }
@@ -82,14 +78,6 @@ void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
 qint32 main(qint32 argc, char* argv[])
 {
     constexpr QLatin1String appName("kdiff3", sizeof("kdiff3") - 1);
-    /*
-        QTextCodec auto registers text codecs in its constructor.
-        Do this now because we only need one codec object.
-        This object is Qt's domain once created and must be on the heap.
-    */
-    const UTF8BOMCodec *textCodec = new UTF8BOMCodec();
-    Q_UNUSED(textCodec);
-
     //Syncronize qt HDPI behavoir on all versions/platforms
     QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
     QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
@@ -103,13 +91,11 @@ qint32 main(qint32 argc, char* argv[])
     const QString i18nName = i18n("KDiff3");
     QString appVersion(KDIFF3_VERSION_STRING);
 
-    if(sizeof(void*) == 8)
-        appVersion += i18nc("Program version info.", " (64 bit)");
-    else if(sizeof(void*) == 4)
-        appVersion += i18nc("Program version info.", " (32 bit)");
+    appVersion += i18nc("Program version info.", " (64 bit)");
+
     const QString description = i18n("Tool for Comparison and Merge of Files and Folders");
     const QString copyright = i18n("(c) 2002-2014 Joachim Eibl, (c) 2017 Michael Reeves KF5/Qt5 port");
-    const QString homePage = QStringLiteral("https://kde.org/applications/development/kdiff3");
+    const QString homePage = QStringLiteral("https://apps.kde.org/kdiff3");
 
     KAboutData aboutData(appName, i18nName,
                          appVersion, description, KAboutLicense::GPL_V2, copyright, QString(),
@@ -195,12 +181,21 @@ qint32 main(qint32 argc, char* argv[])
     aboutData.processCommandLine(cmdLineParser);
 
     /*
-      Do not attempt to call show here that will be done later.
-      This variable exists solely to insure the KDiff3Shell is deleted on exit.
+        This short segment is wrapped in a lambda to delay KDiff3Shell construction until
+        after the main event loop starts. Thus allowing us to avoid std::exit as much as
+        possiable. Makes for a cleaner exit.
     */
-    const QPointer<KDiff3Shell> p(new KDiff3Shell());//QPointer will take it from here.
-    Q_UNUSED(p);
-
+    QPointer<KDiff3Shell> p;
+    QMetaObject::invokeMethod(
+        qApp, [&p] {
+            /*
+              Do not attempt to call show here that will be done later.
+              This variable exists solely to insure the KDiff3Shell is deleted on exit.
+            */
+            p = new KDiff3Shell();
+        },
+        Qt::QueuedConnection);
     qint32 retVal = QApplication::exec();
+    delete p;
     return retVal;
 }
