@@ -10,9 +10,9 @@
 
 #include "progress.h"
 
-#include "common.h"
 #include "defmac.h"
 #include "difftextwindow.h"
+#include "TypeUtils.h"
 
 #include <cmath>
 
@@ -43,7 +43,7 @@ ProgressDialog::ProgressDialog(QWidget* pParent, QStatusBar* pStatusBar)
     //Abort if verticalLayout is not the immediate child of the dialog. This interferes with re-sizing.
     assert(dialogUi.layout->parent() == this);
 
-    chk_connect(dialogUi.abortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
+    chk_connect_a(dialogUi.abortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
     if(m_pStatusBar != nullptr)
     {
         m_pStatusBarWidget = new QWidget;
@@ -54,7 +54,7 @@ ProgressDialog::ProgressDialog(QWidget* pParent, QStatusBar* pStatusBar)
         m_pStatusProgressBar->setRange(0, 1000);
         m_pStatusProgressBar->setTextVisible(false);
         m_pStatusAbortButton = new QPushButton(i18n("&Cancel"));
-        chk_connect(m_pStatusAbortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
+        chk_connect_a(m_pStatusAbortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
         pStatusBarLayout->addWidget(m_pStatusProgressBar);
         pStatusBarLayout->addWidget(m_pStatusAbortButton);
         m_pStatusBar->addPermanentWidget(m_pStatusBarWidget, 0);
@@ -63,10 +63,9 @@ ProgressDialog::ProgressDialog(QWidget* pParent, QStatusBar* pStatusBar)
     }
 
     resize(400, 100);
-#ifndef AUTOTEST
+
     m_t1.start();
     m_t2.start();
-#endif
 
     initConnections();
 }
@@ -106,7 +105,6 @@ void ProgressDialog::initConnections()
 
 void ProgressDialog::setStayHidden(bool bStayHidden)
 {
-#ifndef AUTOTEST
     if(m_bStayHidden != bStayHidden)
     {
         m_bStayHidden = bStayHidden;
@@ -124,12 +122,9 @@ void ProgressDialog::setStayHidden(bool bStayHidden)
             else
                 hideStatusBarWidget(); // delayed
         }
-        if(isVisible() && m_bStayHidden)
+        if(m_bStayHidden)
             hide(); // delayed hide
     }
-#else
-    Q_UNUSED(bStayHidden);
-#endif
 }
 
 void ProgressDialog::push()
@@ -143,10 +138,10 @@ void ProgressDialog::push()
     else
     {
         m_bWasCancelled = false;
-#ifndef AUTOTEST
+
         m_t1.restart();
         m_t2.restart();
-#endif
+
         if(!m_bStayHidden)
             show();
     }
@@ -158,10 +153,8 @@ void ProgressDialog::beginBackgroundTask()
 {
     if(backgroundTaskCount > 0)
     {
-#ifndef AUTOTEST
         m_t1.restart();
         m_t2.restart();
-#endif
     }
     backgroundTaskCount++;
     if(!m_bStayHidden)
@@ -213,7 +206,7 @@ void ProgressDialog::setInformation(const QString& info, bool bRedrawUpdate)
     recalc(bRedrawUpdate);
 }
 
-void ProgressDialog::setMaxNofSteps(const qint64 maxNofSteps)
+void ProgressDialog::setMaxNofSteps(const quint64 maxNofSteps)
 {
     if(m_progressStack.empty() || maxNofSteps == 0)
         return;
@@ -225,10 +218,9 @@ void ProgressDialog::setMaxNofSteps(const qint64 maxNofSteps)
 
 void ProgressDialog::setInformationImp(const QString& info)
 {
-#ifndef AUTOTEST
     assert(!m_progressStack.empty());
 
-    qint32 level = m_progressStack.size();
+    size_t level = m_progressStack.size();
     if(level == 1)
     {
         dialogUi.information->setText(info);
@@ -240,12 +232,9 @@ void ProgressDialog::setInformationImp(const QString& info)
     {
         dialogUi.subInformation->setText(info);
     }
-#else
-    Q_UNUSED(info);
-#endif
 }
 
-void ProgressDialog::addNofSteps(const qint64 nofSteps)
+void ProgressDialog::addNofSteps(const quint64 nofSteps)
 {
     if(m_progressStack.empty())
         return;
@@ -264,7 +253,7 @@ void ProgressDialog::step(bool bRedrawUpdate)
     recalc(bRedrawUpdate);
 }
 
-void ProgressDialog::setCurrent(qint64 subCurrent, bool bRedrawUpdate)
+void ProgressDialog::setCurrent(quint64 subCurrent, bool bRedrawUpdate)
 {
     if(m_progressStack.empty())
         return;
@@ -277,7 +266,7 @@ void ProgressDialog::setCurrent(qint64 subCurrent, bool bRedrawUpdate)
 
 void ProgressDialog::clear()
 {
-    if(m_progressStack.isEmpty())
+    if(m_progressStack.empty())
         return;
 
     ProgressLevelData& pld = m_progressStack.back();
@@ -311,7 +300,6 @@ void ProgressDialog::setSubRangeTransformation(double dMin, double dMax)
 
 void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
 {
-#ifndef AUTOTEST
     m_pJob = pJob;
     m_currentJobInfo = jobInfo;
     dialogUi.slowJobInfo->setText(m_currentJobInfo);
@@ -335,27 +323,20 @@ void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
     {
         m_eventLoop->processEvents(QEventLoop::WaitForMoreEvents);
     }
-#else
-    Q_UNUSED(pJob);
-    Q_UNUSED(jobInfo);
-#endif
 }
 
 void ProgressDialog::exitEventLoop()
 {
-#ifndef AUTOTEST
     if(m_progressDelayTimer)
         killTimer(m_progressDelayTimer);
     m_progressDelayTimer = 0;
     m_pJob = nullptr;
     if(m_eventLoop != nullptr)
         m_eventLoop->exit();
-#endif
 }
 
 void ProgressDialog::recalc(bool bUpdate)
 {
-#ifndef AUTOTEST
     if(!m_bWasCancelled)
     {
         if(QThread::currentThread() == m_pGuiThread)
@@ -366,7 +347,7 @@ void ProgressDialog::recalc(bool bUpdate)
             if(!m_bStayHidden)
                 m_progressDelayTimer = startTimer(3000); /* 3 s delay */
 
-            qint32 level = m_progressStack.size();
+            size_t level = m_progressStack.size();
             if((bUpdate && level == 1) || m_t1.elapsed() > 200)
             {
                 if(m_progressStack.empty())
@@ -376,20 +357,20 @@ void ProgressDialog::recalc(bool bUpdate)
                 }
                 else
                 {
-                    QList<ProgressLevelData>::iterator i = m_progressStack.begin();
-                    qint32 value = qint32(1000.0 * (getAtomic(i->m_current) * (i->m_dRangeMax - i->m_dRangeMin) / getAtomic(i->m_maxNofSteps) + i->m_dRangeMin));
+                    std::list<ProgressLevelData>::iterator i = m_progressStack.begin();
+                    qint32 value = qint32(1000.0 * (i->m_current.loadRelaxed() * (i->m_dRangeMax - i->m_dRangeMin) / i->m_maxNofSteps.loadRelaxed() + i->m_dRangeMin));
                     dialogUi.progressBar->setValue(value);
                     if(m_bStayHidden && m_pStatusProgressBar)
                         m_pStatusProgressBar->setValue(value);
 
                     ++i;
                     if(i != m_progressStack.end())
-                        dialogUi.subProgressBar->setValue((qint32)lround(1000.0 * (getAtomic(i->m_current) * (i->m_dRangeMax - i->m_dRangeMin) / getAtomic(i->m_maxNofSteps) + i->m_dRangeMin)));
+                        dialogUi.subProgressBar->setValue((qint32)lround(1000.0 * (i->m_current.loadRelaxed() * (i->m_dRangeMax - i->m_dRangeMin) / i->m_maxNofSteps.loadRelaxed() + i->m_dRangeMin)));
                     else
                         dialogUi.subProgressBar->setValue((qint32)lround(1000.0 * m_progressStack.front().m_dSubRangeMin));
                 }
 
-                if(!m_bStayHidden && !isVisible())
+                if(!m_bStayHidden)
                     show();
                 qApp->processEvents();
                 m_t1.restart();
@@ -400,30 +381,24 @@ void ProgressDialog::recalc(bool bUpdate)
             QMetaObject::invokeMethod(this, "recalc", Qt::QueuedConnection, Q_ARG(bool, bUpdate));
         }
     }
-#else
-    Q_UNUSED(bUpdate);
-#endif
 }
 
 void ProgressDialog::show()
 {
-#ifndef AUTOTEST
     if(m_progressDelayTimer)
         killTimer(m_progressDelayTimer);
     if(m_delayedHideTimer)
         killTimer(m_delayedHideTimer);
     m_progressDelayTimer = 0;
     m_delayedHideTimer = 0;
-    if(!isVisible() && (parentWidget() == nullptr || parentWidget()->isVisible()))
+    if(parentWidget() == nullptr || parentWidget()->isVisible())
     {
         QDialog::show();
     }
-#endif
 }
 
 void ProgressDialog::hide()
 {
-#ifndef AUTOTEST
     if(m_progressDelayTimer)
         killTimer(m_progressDelayTimer);
     m_progressDelayTimer = 0;
@@ -431,12 +406,10 @@ void ProgressDialog::hide()
     if(m_delayedHideTimer)
         killTimer(m_delayedHideTimer);
     m_delayedHideTimer = startTimer(100);
-#endif
 }
 
 void ProgressDialog::delayedHide()
 {
-#ifndef AUTOTEST
     if(m_pJob != nullptr)
     {
         m_pJob->kill(KJob::Quietly);
@@ -451,21 +424,17 @@ void ProgressDialog::delayedHide()
     dialogUi.subProgressBar->setValue(0);
     dialogUi.subInformation->setText("");
     dialogUi.slowJobInfo->setText("");
-#endif
 }
 
 void ProgressDialog::hideStatusBarWidget()
 {
-#ifndef AUTOTEST
     if(m_delayedHideStatusBarWidgetTimer)
         killTimer(m_delayedHideStatusBarWidgetTimer);
     m_delayedHideStatusBarWidgetTimer = startTimer(100);
-#endif
 }
 
 void ProgressDialog::delayedHideStatusBarWidget()
 {
-#ifndef AUTOTEST
     if(m_progressDelayTimer)
         killTimer(m_progressDelayTimer);
     m_progressDelayTimer = 0;
@@ -475,27 +444,21 @@ void ProgressDialog::delayedHideStatusBarWidget()
         m_pStatusProgressBar->setValue(0);
         m_pStatusBar->clearMessage();
     }
-#endif
 }
 
 void ProgressDialog::reject()
 {
-#ifndef AUTOTEST
     cancel(eUserAbort);
     QDialog::reject();
-#endif
 }
 
 void ProgressDialog::slotAbort()
 {
-#ifndef AUTOTEST
     reject();
-#endif
 }
 
 bool ProgressDialog::wasCancelled()
 {
-#ifndef AUTOTEST
     if(QThread::currentThread() == m_pGuiThread)
     {
         if(m_t2.elapsed() > 100)
@@ -505,21 +468,15 @@ bool ProgressDialog::wasCancelled()
         }
     }
     return m_bWasCancelled;
-#else
-    return false;
-#endif
 }
 
 void ProgressDialog::clearCancelState()
 {
-#ifndef AUTOTEST
     m_bWasCancelled = false;
-#endif
 }
 
 void ProgressDialog::cancel(e_CancelReason eCancelReason)
 {
-#ifndef AUTOTEST
     if(!m_bWasCancelled)
     {
         m_bWasCancelled = true;
@@ -527,9 +484,6 @@ void ProgressDialog::cancel(e_CancelReason eCancelReason)
         if(m_eventLoop != nullptr)
             m_eventLoop->exit(1);
     }
-#else
-    Q_UNUSED(eCancelReason);
-#endif
 }
 
 ProgressDialog::e_CancelReason ProgressDialog::cancelReason()
@@ -539,10 +493,9 @@ ProgressDialog::e_CancelReason ProgressDialog::cancelReason()
 
 void ProgressDialog::timerEvent(QTimerEvent* te)
 {
-#ifndef AUTOTEST
     if(te->timerId() == m_progressDelayTimer)
     {
-        if(!isVisible() && !m_bStayHidden)
+        if(!m_bStayHidden)
         {
             show();
         }
@@ -560,7 +513,4 @@ void ProgressDialog::timerEvent(QTimerEvent* te)
         m_delayedHideStatusBarWidgetTimer = 0;
         delayedHideStatusBarWidget();
     }
-#else
-    Q_UNUSED(te);
-#endif
 }
